#include "oled.h"
#include "linux/i2c-dev.h"
#include "linux/i2c.h"
#include "u8g2/u8g2.h"
#include <global.h>
#include <linux/fcntl.h>
#include <stdio.h>
int fd;
unsigned char i2c_init()
{
    const char *i2c_dev = "/dev/i2c-2";
    fd = open(i2c_dev, O_RDWR);
    if (fd < 0)
    {
        printf("not have /dev/i2c-2 t\r\n");
        fflush(stdout);
        return -1;
    }
    return 0;
}
// I2C Addr 7bit
unsigned char base_i2c_write(unsigned char device_addr, unsigned char *buff, int num)
{

    struct i2c_rdwr_ioctl_data data;
    struct i2c_msg msgs_str[1];
    data.msgs = msgs_str;
    data.nmsgs = 1;
    data.msgs[0].len = num;
    data.msgs[0].addr = device_addr;
    data.msgs[0].flags = 0;
    data.msgs[0].buf = buff;
    ioctl(fd, I2C_RDWR, (unsigned long)&data);
}
unsigned char base_i2c_read(unsigned char device_addr, unsigned char *buff, int num)
{
    int fd, ret;
    struct i2c_rdwr_ioctl_data i2c_data;
    const char *i2c_dev = "/dev/i2c-0";
    fd = open(i2c_dev, O_RDWR);
    if (fd < 0)
    {
        printf("not have /dev/i2c-0 t\r\n");
        fflush(stdout);
        return -1;
    }

    i2c_data.nmsgs = 1;
    i2c_data.msgs = (struct i2c_msg *)malloc(i2c_data.nmsgs * sizeof(struct i2c_msg));
    if (i2c_data.msgs == NULL)
    {
        printf("malloc error");
        fflush(stdout);
        close(fd);
        return -1;
    }
    ioctl(fd, I2C_TIMEOUT, 1);
    ioctl(fd, I2C_RETRIES, 2);
    i2c_data.msgs[0].len = num;
    i2c_data.msgs[0].addr = device_addr;
    i2c_data.msgs[0].flags = 1; // 0: write 1:read
    i2c_data.msgs[0].buf = buff;

    ret = ioctl(fd, I2C_RDWR, (unsigned long)&i2c_data);
    if (ret < 0)
    {
        printf("read data %x  error\r\n", device_addr);
        close(fd);
        free(i2c_data.msgs);
        return 1;
    }
    free(i2c_data.msgs);
    close(fd);

    int i;
    printf(" Begin print read data \n");
    for (i = 0; i < num; i++)
    {
        printf(" 0x%02x", buff[i]);
    }
    printf("\n");
    return 0;
}
uint8_t u8x8_byte_hw_i2c(u8x8_t *u8x8, uint8_t msg, uint8_t arg_int, void *arg_ptr)
{
    /* u8g2/u8x8 will never send more than 32 bytes between START_TRANSFER and END_TRANSFER */
    static uint8_t buffer[128];
    static uint8_t buf_idx;
    uint8_t *data;

    switch (msg)
    {
    case U8X8_MSG_BYTE_INIT:
    {
        /* add your custom code to init i2c subsystem */
        // I2C1_Init(); // I2C初始化
    }
    break;

    case U8X8_MSG_BYTE_START_TRANSFER:
    {
        buf_idx = 0;
    }
    break;

    case U8X8_MSG_BYTE_SEND:
    {
        data = (uint8_t *)arg_ptr;

        while (arg_int > 0)
        {
            buffer[buf_idx++] = *data;
            data++;
            arg_int--;
        }
    }
    break;

    case U8X8_MSG_BYTE_END_TRANSFER:
    {
        if (base_i2c_write(OLED_ADDRESS, buffer, buf_idx) != 0)
            return 0;
    }
    break;

    case U8X8_MSG_BYTE_SET_DC:
        break;

    default:
        return 0;
    }

    return 1;
}

uint8_t u8x8_gpio_and_delay(u8x8_t *u8x8, uint8_t msg, uint8_t arg_int, void *arg_ptr)
{
    switch (msg)
    {
    case U8X8_MSG_DELAY_100NANO: // delay arg_int * 100 nano seconds
        break;
    case U8X8_MSG_DELAY_10MICRO: // delay arg_int * 10 micro seconds
        break;
    case U8X8_MSG_DELAY_MILLI: // delay arg_int * 1 milli second
        break;
    case U8X8_MSG_DELAY_I2C:      // arg_int is the I2C speed in 100KHz, e.g. 4 = 400 KHz
        break;                    // arg_int=1: delay by 5us, arg_int = 4: delay by 1.25us
    case U8X8_MSG_GPIO_I2C_CLOCK: // arg_int=0: Output low at I2C clock pin
        break;                    // arg_int=1: Input dir with pullup high for I2C clock pin
    case U8X8_MSG_GPIO_I2C_DATA:  // arg_int=0: Output low at I2C data pin
        break;                    // arg_int=1: Input dir with pullup high for I2C data pin
    case U8X8_MSG_GPIO_MENU_SELECT:
        u8x8_SetGPIOResult(u8x8, /* get menu select pin state */ 0);
        break;
    case U8X8_MSG_GPIO_MENU_NEXT:
        u8x8_SetGPIOResult(u8x8, /* get menu next pin state */ 0);
        break;
    case U8X8_MSG_GPIO_MENU_PREV:
        u8x8_SetGPIOResult(u8x8, /* get menu prev pin state */ 0);
        break;
    case U8X8_MSG_GPIO_MENU_HOME:
        u8x8_SetGPIOResult(u8x8, /* get menu home pin state */ 0);
        break;
    default:
        u8x8_SetGPIOResult(u8x8, 1); // default return value
        break;
    }
    return 1;
}
void u8g2Init(u8g2_t *u8g2)
{
    i2c_init();
    // u8g2_Setup_ssd1306_i2c_128x64_noname_f(u8g2, U8G2_R0, u8x8_byte_hw_i2c, u8x8_gpio_and_delay); // 初始化 u8g2 结构体
    u8g2_Setup_sh1106_i2c_128x64_noname_f(u8g2, U8G2_R0, u8x8_byte_hw_i2c, u8x8_gpio_and_delay); // 初始化 u8g2 结构体
    u8g2_InitDisplay(u8g2); // 根据所选的芯片进行初始化工作，初始化完成后，显示器处于关闭状态
    // u8g2_ClearBuffer(u8g2);
    u8g2_SetPowerSave(u8g2, 0); // 打开显示器
    LOG_D("1.3\" oled init ok!\n");
}
void power_off(u8g2_t *u8g2, uint8_t flag)
{
    u8g2_ClearBuffer(u8g2);        //
    u8g2_SetPowerSave(u8g2, flag); // 显示器
}
